@@ -1,6 +1,6 @@
 /*
  * repmgrd.c - Replication manager daemon
- * Copyright (C) 2ndQuadrant, 2010-2015
+ * Copyright (C) 2ndQuadrant, 2010-2016
  *
  * This module connects to the nodes of a replication cluster and monitors
  * how far are they from master
@@ -79,7 +79,6 @@ static void do_master_failover(void);
 static bool do_upstream_standby_failover(t_node_info upstream_node);
 
 static t_node_info get_node_info(PGconn *conn, char *cluster, int node_id);
-static t_server_type parse_node_type(const char *type);
 static XLogRecPtr lsn_to_xlogrecptr(char *lsn, bool *format_ok);
 
 /*
@@ -143,6 +142,20 @@ main(int argc, char **argv)
 
 	set_progname(argv[0]);
 
+	/* Disallow running as root to prevent directory ownership problems */
+	if (geteuid() == 0)
+	{
+		fprintf(stderr,
+				_("%s: cannot be run as root\n"
+				  "Please log in (using, e.g., \"su\") as the "
+				  "(unprivileged) user that owns "
+				  "the data directory.\n"
+				),
+				progname());
+		exit(1);
+	}
+
+
 	while ((c = getopt_long(argc, argv, "?Vf:vmdp:", long_options, &optindex)) != -1)
 	{
 		switch (c)
@@ -1322,7 +1335,7 @@ do_master_failover(void)
 						log_crit(
 							_("unable to obtain LSN from node %i"), nodes[i].node_id
 							);
-						log_info(
+						log_hint(
 							_("please check that 'shared_preload_libraries=repmgr_funcs' is set in postgresql.conf\n")
 							);
 
@@ -1479,7 +1492,7 @@ do_master_failover(void)
 		/* wait */
 		sleep(10);
 
-		log_info(_("node %d is the best candidate for new master, attempting to follow...\n"),
+		log_notice(_("node %d is the best candidate for new master, attempting to follow...\n"),
 				 best_candidate.node_id);
 
 		/*
@@ -1525,11 +1538,11 @@ do_master_failover(void)
 
 		node_info = get_node_info(new_master_conn, local_options.cluster_name, local_options.node);
 		appendPQExpBuffer(&event_details,
-						  _("Node %i now following new upstream node %i"),
+						  _("node %i now following new upstream node %i"),
 						  node_info.node_id,
 						  best_candidate.node_id);
 
-		log_info("%s\n", event_details.data);
+		log_notice("%s\n", event_details.data);
 
 		create_event_record(new_master_conn,
 							&local_options,
@@ -1685,7 +1698,7 @@ do_upstream_standby_failover(t_node_info upstream_node)
 	}
 
 	appendPQExpBuffer(&event_details,
-					  _("Node %i is now following upstream node %i"),
+					  _("node %i is now following upstream node %i"),
 					  node_info.node_id,
 					  upstream_node_id);
 
@@ -2254,13 +2267,13 @@ check_and_create_pid_file(const char *pid_file)
 t_node_info
 get_node_info(PGconn *conn, char *cluster, int node_id)
 {
-	PGresult   *res;
+	int res;
 
 	t_node_info node_info = T_NODE_INFO_INITIALIZER;
 
-	res = get_node_record(conn, cluster, node_id);
+	res = get_node_record(conn, cluster, node_id, &node_info);
 
-	if (PQresultStatus(res) != PGRES_TUPLES_OK)
+	if (res == -1)
 	{
 		PQExpBufferData errmsg;
 		initPQExpBuffer(&errmsg);
@@ -2279,47 +2292,14 @@ get_node_info(PGconn *conn, char *cluster, int node_id)
 							false,
 							errmsg.data);
 
-		PQclear(res);
+		PQfinish(conn);
 		terminate(ERR_DB_QUERY);
 	}
 
-	if (!PQntuples(res)) {
+	if (res == 0)
+	{
 		log_warning(_("No record found record for node %i\n"), node_id);
-		PQclear(res);
-		node_info.node_id = NODE_NOT_FOUND;
-		return node_info;
 	}
 
-	node_info.node_id = atoi(PQgetvalue(res, 0, 0));
-	node_info.upstream_node_id = atoi(PQgetvalue(res, 0, 1));
-	strncpy(node_info.conninfo_str, PQgetvalue(res, 0, 2), MAXLEN);
-	node_info.type = parse_node_type(PQgetvalue(res, 0, 3));
-	strncpy(node_info.slot_name, PQgetvalue(res, 0, 4), MAXLEN);
-	node_info.active = (strcmp(PQgetvalue(res, 0, 5), "t") == 0)
-		? true
-		: false;
-
-	PQclear(res);
-
 	return node_info;
 }
-
-
-static t_server_type
-parse_node_type(const char *type)
-{
-	if (strcmp(type, "master") == 0)
-	{
-		return MASTER;
-	}
-	else if (strcmp(type, "standby") == 0)
-	{
-		return STANDBY;
-	}
-	else if (strcmp(type, "witness") == 0)
-	{
-		return WITNESS;
-	}
-
-	return UNKNOWN;
-}
